@@ -93,16 +93,7 @@ class ZodbInfoView(BrowserView):
                 transaction.abort()
 
     def render(self):
-        self.obj = None
-
-        if 'oid' not in self.request:
-            self.obj = self.findClosestPersistent()
-
-        if self.obj is None:
-            oid = p64(int(self.request.get('oid', self.getRootOid()), 0))
-            jar = self.jar()
-            self.obj = jar.get(oid)
-
+        self.obj = self.selectObjectToView()
         self.history = IObjectHistory(self.obj)
         self.latest = True
         if self.request.get('tid'):
@@ -134,6 +125,18 @@ class ZodbInfoView(BrowserView):
     def _redirectToSelf(self):
         self.request.response.redirect(self.getUrl())
 
+    def selectObjectToView(self):
+        obj = None
+        if 'oid' not in self.request:
+            obj = self.findClosestPersistent()
+        if obj is None:
+            if 'oid' in self.request:
+                oid = int(self.request['oid'], 0)
+            else:
+                oid = self.getRootOid()
+            obj = self.jar().get(p64(oid))
+        return obj
+
     def findClosestPersistent(self):
         obj = removeSecurityProxy(self.context)
         while not isinstance(obj, Persistent):
@@ -159,7 +162,11 @@ class ZodbInfoView(BrowserView):
         return self.state.getObjectId()
 
     def getObjectType(self):
-        return str(getattr(self.obj, '__class__', None))
+        cls = getattr(self.obj, '__class__', None)
+        if type(self.obj) is not cls:
+            return '%s - %s' % (type(self.obj), cls)
+        else:
+            return str(cls)
 
     def getStateTid(self):
         return u64(self.state.tid)
@@ -182,9 +189,9 @@ class ZodbInfoView(BrowserView):
         try:
             return self.request.annotations['ZODB.interfaces.IConnection']
         except KeyError:
-            obj = removeSecurityProxy(self.context)
-            while not isinstance(obj, Persistent):
-                obj = removeSecurityProxy(obj.__parent__)
+            obj = self.findClosestPersistent()
+            if obj is None:
+                raise Exception("ZODB connection not available for this request")
             return obj._p_jar
 
     def locate(self, path):
